//    File:    binarySearchTree.h
//    Author:  
//    Date:    
//    Course:  IE 171, Lab 5
//    Purpose: This is the declaration file for a binary tree.
//#############################################################################
//#############################################################################

#ifndef BINARY_SEARCH_TREE_H
#define BINARY_SEARCH_TREE_H

#include <iostream>

//#############################################################################
/** Key and data type. */
typedef int          KEY;
typedef double       DATA;    // Not used in this Lab

//#############################################################################
/** The node of binary tree. */
//#############################################################################

class BinaryNode
{
 private:
  KEY        key_;           // Key used to sort
  DATA       data_;          // Data stored  // Not used
  int        depth_;         // The depth of this node
  BinaryNode * left_, * right_, * parent_; // Pointers to children & parent 


  // Fill in other needed data

 public:
  BinaryNode()               // Modify if needed
    : key_(0), data_(0), depth_(0), left_(0), right_(0), parent_(0) {}
  BinaryNode(KEY k)          // Modify if needed
    : key_(k), data_(0), depth_(0), left_(0), right_(0), parent_(0) {}
  BinaryNode(KEY k, DATA d)  // Modify if needed
    : key_(k), data_(d), depth_(0), left_(0), right_(0), parent_(0) {}
  ~BinaryNode() {}

  // ************ Const Query Functions *****************

  KEY         getKey() const              { return key_; }
  DATA        getData() const             { return data_; }
  int         getDepth() const            { return depth_; }
  BinaryNode* getLeft() const             { return left_; }
  BinaryNode* getRight() const            { return right_; }
  BinaryNode* getParent() const           { return parent_; }

  // ************ Modification Functions *****************

  void        setKey(KEY k)               { key_ = k; }
  void        setData(DATA d)             { data_ = d; }
  void        setDepth(int d)             { depth_ = d; }
  void        setLeft(BinaryNode* bn)     { left_ = bn; }
  void        setRight(BinaryNode* bn)    { right_ = bn; }
  void        setParent(BinaryNode* bn)   { parent_ = bn; }

  // Fill in other needed modification functions

};


//#############################################################################
/** Declaration of a binary tree.*/
//#############################################################################

class BinarySearchTree
{
 private:
  
   BinaryNode* root_;  // The root of the binary tree
   int size_;          // The size of the binary tree

 private:
  //---------------------------------------------------------------------------
  //  These functions are private in order to simplify the interface. 
  //  The public functions can only be called on the tree as a whole.
  //  For implementation purposes, we may need to be able to call these 
  //  functions (for recursion), but only from within the class.

  // Counts the depth of the tree.
  int          getDepth_(BinaryNode* p) const;

  // Calculate the sum of the depths of nodes in the subtree rooted at p.
  // Used for finding the average depth of the tree.
  void         getTotalDepth_(BinaryNode* x, int& total ) const;

  // Print out keys&data in sorted order.
  void         printSorted_(BinaryNode* x, std::ostream& os) const;

  // Search for a node with a given key recursively in subtree with root x.
  BinaryNode*  search_(BinaryNode* x, const KEY k) const;

  // Delete the whole subtree with root x. Needed for destructor.
  void         deleteTree_(BinaryNode* p);

  //---------------------------------------------------------------------------

 public: 
  
  BinarySearchTree() : root_(0), size_(0) {}
  ~BinarySearchTree() { deleteTree(); }         
  
 public:
  //---------------------------------------------------------------------------
  // These are the public functions that operate on the whole tree.
 
  // ************ Const Query Functions *****************

  // Return the size of tree.
  int          getSize() const;

  // Returns the depth of the whole tree.
  int          getDepth() const;
  
  // Returns the average depth of all nodes in the tree.
  double       getAverageDepth() const;
  
  // Print out keys & data in sorted order.
  void         printSorted(std::ostream& os) const;

  // Search for a node with given key.
  BinaryNode*  search(const KEY k) const;
 
  // ************ Modification Functions *****************

  // Build a binary tree from istream.
  void         buildTree(std::istream& is);

  // Build a binary tree from a given array.
  void         buildTree(KEY* element, const int num);

  // Delete the whole tree.
  void         deleteTree();

  // Delete a given node.
  bool         deleteNode(BinaryNode* z);

  // Insert a given node.
  void         insert(BinaryNode* z); 
  
  //---------------------------------------------------------------------------
};
#endif        
